# frozen_string_literal: true

module BrocadeVRouter
  class Configuration
    ParseError = Class.new(StandardError)

    LTSEP = ' '.freeze
    OBRKT = '{'.freeze
    CBRKT = '}'.freeze
    ARRAY_FIELDS = {
      'address' => 'address-group',
      'port' => 'port-group'
    }.freeze

    def initialize(data)
      @conf = parse_lines! data.lines, []
    end

    def to_h
      @conf
    end

    private

    def parse_lines!(lines, path)
      conf = {}
      while lines.any?
        begin
          literals = lines.shift.split(LTSEP)
          name = literals.shift

          if name == CBRKT
            return conf
          elsif literals.last == OBRKT
            if literals.first != OBRKT
              conf[name] ||= {}
              conf[name][literals.first] = parse_lines! lines, path + [name, literals.first]
            else
              conf[name] = parse_lines! lines, path + [name]
            end
          else
            value = literals.join(LTSEP).tr('\'"', '')
            if array_field?(path, name)
              conf[name] ||= []
              conf[name] << value
            else
              conf[name] = value.empty? ? nil : value
            end
          end
        rescue StandardError => e
          raise ParseError.new("#{e.message}: #{literals}")
        end
      end

      conf
    end

    def array_field?(path, name)
      ARRAY_FIELDS[name] && path.include?(ARRAY_FIELDS[name])
    end
  end
end
